@@ -339,7 +339,7 @@ _dbus_g_value_types_init (void)
 /**
  * Get the GLib type ID for a DBusGObjectPath boxed type.
  *
- * @returns GLib type
+ * Returns: GLib type
  */
 GType
 dbus_g_object_path_get_g_type (void)
@@ -1139,18 +1139,18 @@ _dbus_gvalue_demarshal_message  (DBusGValueMarshalCtx    *context,
   GValueArray *ret;
   DBusMessageIter iter;
   int current_type;
-  guint index;
+  guint index_;
   
   ret = g_value_array_new (6);  /* 6 is a typical maximum for arguments */
 
   dbus_message_iter_init (message, &iter);
-  index = 0;
+  index_ = 0;
   while ((current_type = dbus_message_iter_get_arg_type (&iter)) != DBUS_TYPE_INVALID)
     {
       GValue *value;
       GType gtype;
 
-      if (index >= n_types)
+      if (index_ >= n_types)
 	{
 	  g_set_error (error, DBUS_GERROR,
 		       DBUS_GERROR_INVALID_ARGS,
@@ -1159,17 +1159,17 @@ _dbus_gvalue_demarshal_message  (DBusGValueMarshalCtx    *context,
 	}
       
       g_value_array_append (ret, NULL);
-      value = g_value_array_get_nth (ret, index);
+      value = g_value_array_get_nth (ret, index_);
 
-      gtype = types[index]; 
+      gtype = types[index_]; 
       g_value_init (value, gtype);
 
       if (!_dbus_gvalue_demarshal (context, &iter, value, error))
 	goto lose;
       dbus_message_iter_next (&iter);
-      index++;
+      index_++;
     }
-  if (index < n_types)
+  if (index_ < n_types)
     {
       g_set_error (error, DBUS_GERROR,
 		   DBUS_GERROR_INVALID_ARGS,
@@ -1827,8 +1827,10 @@ assert_bidirectional_mapping (GType gtype, const char *expected_sig)
 
 /**
  * @ingroup DBusGLibInternals
+ * @test_data_dir:
+ *
  * Unit test for general glib stuff
- * @returns #TRUE on success.
+ * Returns: #TRUE on success.
  */
 gboolean
 _dbus_gvalue_test (const char *test_data_dir)
